;
;   Copyright 1991 Inference Corporation
;   550 North Continental Blvd.
;   El Segundo, CA 90245
;   An Unpublished Work - All Rights Reserved.
;   CBR Express (TM) Patent Pending.
;
;   RESTRICTED RIGHTS LEGEND
;   Use, disclosure or reproduction of this software is subject to the
;   conditions contained in the Inference Software License Agreement.
;   This software may only be used in conjunction with ART-IM and
;   CBR Express.
;   
;   Use, reproduction or disclosure by the U.S. Government is subject
;   to restriction as set forth in the Rights in Technical Data and
;   Computer Software clause at 52.227-19.
;
;   The program contained herein is offered solely for the convenience
;   of Inference licensees and may be modified by licensee for use with
;   Inference programs.  The program is provided "as-is" and is not
;   eligible for Inference technical assistance.  Inference assumes no
;   obligation to maintain or support this program, nor to insure its
;   compatibility with licensee's systems or software.
;   INFERENCE HEREBY DISCLAIMS ANY AND ALL WARRANTIES INCLUDING, WITHOUT
;   LIMITATION, THE WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
;   PARTICULAR PURPOSE.


PAGE,132
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;       LIBENTRY.ASM
;
;       Windows dynamic-link library entry routine
;
;   This module generates a code segment called INIT_TEXT.
;   It initializes the local heap if one exists and then calls
;   the C routine LibMain() which should have the form:
;   BOOL FAR PASCAL LibMain(HANDLE hInstance,
;                           WORD   wDataSeg,
;                           WORD   cbHeap,
;                           LPSTR  lpszCmdLine);
;
;   The result of the call to LibMain is returned to Windows.
;   The C routine should return TRUE if it completes initialization
;   successfully, FALSE if some error occurs.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

include cmacros.inc

externFP <LibMain>               ; the C routine to be called

createSeg INIT_TEXT, INIT_TEXT, BYTE, PUBLIC, CODE
sBegin    INIT_TEXT
assumes CS,INIT_TEXT

?PLM=0                           ; 'C'naming
externA  <_acrtused>             ; ensures that Win DLL startup code
                                 ; is linked

?PLM=1                           ; 'PASCAL' naming
externFP <LocalInit>             ; Windows heap init routine

cProc   LibEntry, <PUBLIC,FAR>   ; entry point into DLL

cBegin
        push    di               ; handle of the module instance
        push    ds               ; library data segment
        push    cx               ; heap size
        push    es               ; command line segment
        push    si               ; command line offset

        ; if we have some heap then initialize it
        jcxz    callc            ; jump if no heap specified

        ; call the Windows function LocalInit() to set up the heap
        ; LocalInit((LPSTR)start, WORD cbHeap);
        xor     ax,ax
        cCall   LocalInit <ds, ax, cx>

        or      ax,ax            ; did it do it ok ?
        jz      error            ; quit if it failed

        ; invoke the C routine to do any special initialization
callc:
        call    LibMain          ; invoke the 'C' routine
                                 ; (result in AX)
        jmp short exit           ; LibMain is responsible for
                                 ; stack clean up

error:
        pop     si               ; clean up stack on a LocalInit error
        pop     es
        pop     cx
        pop     ds
        pop     di

exit:

cEnd

sEnd   INIT_TEXT

end LibEntry
